@@ -22,12 +22,13 @@
 #include "konqcombo.h"
 
 // Qt
-#include <QtGui/QPainter>
-#include <QtGui/QStyle>
-#include <QtGui/QPixmap>
-#include <QtGui/QKeyEvent>
-#include <QtGui/QItemDelegate>
-#include <QtGui/QListWidgetItem>
+#include <QPainter>
+#include <QStyle>
+#include <QDrag>
+#include <QPixmap>
+#include <QKeyEvent>
+#include <QItemDelegate>
+#include <QListWidgetItem>
 #include <QtCore/QEvent>
 
 // KDE
